# © 2017-2019, ETH Zurich, Institut für Theoretische Physik
# Author: Dominik Gresch <greschd@gmx.ch>
"""
A tool for modifying, comparing and plotting electronic bandstructures.
"""

from . import kpoints
from . import eigenvals
from . import compare
from . import lattice
from . import plot

__version__ = '0.2.1'
